@@ -67,16 +67,9 @@ db_compute.AthenaConnection <- function(con,
                                         ...) {
   db_save_query <- pkg_method("db_save_query", "dplyr")
   in_schema <- pkg_method("in_schema", "dbplyr")
-  
   table <- db_save_query(con, sql, table, ...)
-  if (grepl("\\.", table)) {
-    schema <- gsub("\\..*", "" , table)
-    table <- gsub(".*\\.", "" , table)
-  } else {
-    schema <- con@info$dbms.name
-    table <- table}
-
-  in_schema(schema, table)
+  ll <- db_detect(con, table)
+  in_schema(ll[["dbms.name"]], ll[["table"]])
 }
 
 #' Athena S3 implementation of dbplyr backend functions
@@ -114,7 +107,7 @@ db_save_query.AthenaConnection <- function(con, sql, name ,
                        sql, ";"))
   res <- dbExecute(con, tt_sql)
   dbClearResult(res)
-  name
+  return(name)
 }
 
 #' S3 implementation of \code{db_copy_to} for Athena
