@@ -15,15 +15,19 @@ test_that("Returning meta data",{
   # Test connection is using AWS CLI to set profile_name 
   con = dbConnect(athena())
   
-  res = dbExecute(con, "select * from test_df")
-  res_out = dbHasCompleted(res)
-  res_info = dbGetInfo(res)
-  res_stat = dbStatistics(res)
-  column_info1 = dbColumnInfo(res)
+  res1 = dbExecute(con, "select * from test_df")
+  res2 = dbSendStatement(con, "select * from test_df")
+  res_out2 = dbHasCompleted(res2)
+  res_out1 = dbHasCompleted(res1)
+  res_info = dbGetInfo(res1)
+  res_stat = dbStatistics(res1)
+  column_info1 = dbColumnInfo(res1)
   column_info2 = dbListFields(con, "test_df")
   con_info_exp = names(dbGetInfo(con))
   list_tbl1 = any(grepl("test_df", dbListTables(con, "default")))
   list_tbl2 = nrow(dbGetTables(con, "default")[TableName == "test_df"]) == 1
+  list_tbl3 = nrow(dbGetTables(con)[Schema == "default" & TableName == "test_df"]) == 1
+  list_tbl4 = any(grepl("test_df", dbListTables(con)))
   partition1 = grepl("timestamp", dbGetPartition(con, "test_df")[[1]])
   
   partition2 = names(dbGetPartition(con, "test_df", .format = T)) == "timestamp"
@@ -34,14 +38,19 @@ test_that("Returning meta data",{
   db_show_ddl = gsub(", \n  'transient_lastDdlTime'.*",")", dbShow(con, "test_df"))
   db_info = dbGetInfo(con)
   
-  dbClearResult(res)
+  name1 <- db_detect(con, "table1")
+  name2 <- db_detect(con, "mydatabase.table1")
+
+  dbClearResult(res1)
   dbDisconnect(con)
   
   expect_equal(column_info1, df_col_info)
   expect_equal(column_info2, col_info_exp)
-  expect_equal(con_info, con_info_exp)
+  expect_equal(con_info[order(con_info)], con_info_exp[order(con_info_exp)])
   expect_true(list_tbl1)
   expect_true(list_tbl2)
+  expect_true(list_tbl3)
+  expect_true(list_tbl4)
   expect_true(partition1)
   expect_true(partition2)
   expect_true(partition3)
@@ -51,7 +60,15 @@ test_that("Returning meta data",{
   expect_false(noctua:::is.s3_uri(NULL))
   expect_true(is.list(db_info))
   expect_error(dbGetInfo(con))
-  expect_true(res_out)
-  expect_equal(names(res_info), c("QueryExecutionId", "NextToken"))
+  expect_true(res_out1)
+  expect_true(inherits(res_out2, "logical"))
+  expect_equal(
+    names(res_info), 
+    c("WorkGroup", "StateChangeReason", "OutputLocation", "StatementType",
+      "Query", "Status", "QueryExecutionId", "Statistics"))
   expect_true(is.list(res_stat))
+  expect_error(con_error_msg(res1, "dummy message"), "dummy message")
+  expect_equal(name1, list("dbms.name" = "default", "table" = "table1"))
+  expect_equal(name2, list("dbms.name" = "mydatabase", "table" = "table1"))
 })
+
